/* Copyright (C) 1994 Queen Mary and Westfield College, University of London

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   A copy of the GNU General Public License may be found in
   doc/COPYING in the source tree of this programm.

   This file was written by: Franz Gerschwiler.(goblin@dcs.qmw.ac.uk)

 ****

   modified by: p.hlstede	(peter@calvados.chemie.uni-bremen.de)

   - the code was reorganized and ansified
   - many resource settings were moved to the app-defaults file
   - new items for the setup dialog (word wrap, autoindent)

							06.09.1997
 */

#include <stdio.h>
#include <string.h>

#include <Xm/Xm.h>
#include <Xm/BulletinB.h>
#include <Xm/Frame.h>
#include <Xm/Form.h>
#include <Xm/RowColumn.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/List.h>
#include <Xm/Label.h>
#include <Xm/LabelG.h>
#include <Xm/PushBG.h>
#include <Xm/ToggleB.h>

#include "sys_consts.h"
#include "globals.h"
#include "callbacks.h"
#include "helpEvents.h"
#include "popUps.h"


/***  GLOBAL VARIABLES  ***/
Widget   configPopUp = (Widget)0;
Widget   autosavetimeText, printerText;
Widget   autosaveTB, backupTB, autoindTB, wordwrapTB, fileboxTB;
Widget   helpPopUp = (Widget)0;
Widget   helpLI, helpText;


void  gui_createHelpPopUp(Widget parent)
{
   Arg      args[15];
   int      n;
   Widget   helpFO, helpControlFO, helpListFO, helpLabel, button;


/* CREATE HELP POPUP UNMANAGED */
   helpPopUp = XmCreateBulletinBoardDialog(parent, "Help", NULL, 0);

/* HELP FORM */
   helpFO = XtVaCreateWidget("helpFO", xmFormWidgetClass, helpPopUp, NULL);

/*********************************************************************************/
/* HELP LIST FORM WIDGET */
   helpListFO = XtVaCreateWidget("helpListFO", xmFormWidgetClass, helpFO,
			XmNleftAttachment,   XmATTACH_FORM,
			XmNtopAttachment,    XmATTACH_FORM,
			XmNbottomAttachment, XmATTACH_FORM,
			NULL);

/* HELP LIST LABEL WIDGET */
   helpLabel = XtVaCreateManagedWidget("topics", xmLabelWidgetClass, helpListFO,
			XmNleftAttachment,  XmATTACH_FORM,
			XmNrightAttachment, XmATTACH_FORM,
			XmNtopAttachment,   XmATTACH_FORM,
			NULL);

/* HELP LIST WIDGET */
   n = 0;
   XtSetArg(args[n], XmNvisibleItemCount, 20);              n++;
   XtSetArg(args[n], XmNlistSizePolicy,   XmCONSTANT);      n++;
   XtSetArg(args[n], XmNselectionPolicy,  XmBROWSE_SELECT); n++;
   XtSetArg(args[n], XmNleftAttachment,   XmATTACH_FORM);   n++;
   XtSetArg(args[n], XmNrightAttachment,  XmATTACH_FORM);   n++;
   XtSetArg(args[n], XmNtopWidget,        helpLabel);       n++;
   XtSetArg(args[n], XmNtopAttachment,    XmATTACH_WIDGET); n++;
   helpLI = XmCreateScrolledList(helpListFO, "helpLI", args, n);
   XtAddCallback(helpLI, XmNbrowseSelectionCallback, gui_helpSelectCB, NULL);
   XtManageChild(helpLI);

/* MAIN CONTROL BUTTONS */
   helpControlFO = XtVaCreateWidget("control", xmRowColumnWidgetClass, helpListFO,
			XmNnumColumns,      2,
			XmNorientation,     XmHORIZONTAL,
			XmNtopWidget,       helpLI,
			XmNtopAttachment,   XmATTACH_WIDGET,
			XmNrightAttachment, XmATTACH_FORM,
			XmNleftAttachment,  XmATTACH_FORM,
			NULL);

/* HELP BUTTON WIDGET */
   button = XtVaCreateManagedWidget("onHelp", xmPushButtonGadgetClass, helpControlFO, NULL);
   XtAddCallback(button, XmNdisarmCallback, gui_helpCB, (XtPointer)HELPHELP);

/* CANCEL BUTTON WIDGET */
   button = XtVaCreateManagedWidget("cancel", xmPushButtonGadgetClass, helpControlFO, NULL);
   XtAddCallback(button, XmNdisarmCallback, gui_helpCB, (XtPointer) HELPCANCEL);

   XtManageChild(helpControlFO);
   XtManageChild(helpListFO);

/*************************************************************************/
/* CREATE HELP TEXT CANVAS */
   n = 0;
   XtSetArg(args[n], XmNeditable,              False);             n++;
   XtSetArg(args[n], XmNeditMode,              XmMULTI_LINE_EDIT); n++;
   XtSetArg(args[n], XmNcursorPositionVisible, False);             n++;
   XtSetArg(args[n], XmNleftAttachment,        XmATTACH_WIDGET);   n++;
   XtSetArg(args[n], XmNleftWidget,            helpListFO);        n++;
   XtSetArg(args[n], XmNrightAttachment,       XmATTACH_FORM);     n++;
   XtSetArg(args[n], XmNtopAttachment,         XmATTACH_FORM);     n++;
   XtSetArg(args[n], XmNbottomAttachment,      XmATTACH_FORM);     n++;
   helpText = XmCreateScrolledText(helpFO, "helpText", args, n);
   XtManageChild(helpText);

   gui_createHelpList();

   XtManageChild(helpFO);
}


/**************************************/
/* CREATE CONFIGURATION POPUP         */
/**************************************/
void     gui_createConfigPopUp(Widget parent)
{
   char     txt[20];
   Arg      args[15];
   int      n;
   Widget   autosavetimeFrame, printerFrame, onOffFrame, controlFrame,
            autosavetimeFO, CheckLabel, controlFO, configFO, button;
   Widget   optionCheck;


/* CREATE CONFIG POPUP UNMANAGED */
   configPopUp = XmCreateBulletinBoardDialog(parent, "Configuration", NULL, 0);

/* CONFIG FORM WIDGET */
   configFO = XtVaCreateWidget("configFO", xmFormWidgetClass, configPopUp, NULL);

/*******************************************************************************/
/*                                 OBJECT FRAMES                               */
/*                              MAKES ATTACHMENT EASY                          */
/*******************************************************************************/

/* ON OFF FRAME */
   onOffFrame = XtVaCreateWidget("onOff", xmFrameWidgetClass, configFO,
			XmNtopAttachment,  XmATTACH_FORM,
			XmNleftAttachment, XmATTACH_FORM,
			NULL);

/* CONTROL FRAME */
   controlFrame = XtVaCreateWidget("control", xmFrameWidgetClass, configFO,
			XmNtopAttachment,   XmATTACH_FORM,
			XmNleftAttachment,  XmATTACH_WIDGET,
			XmNleftWidget,      onOffFrame,
			XmNleftOffset,      5,
			XmNrightAttachment, XmATTACH_FORM,
			NULL);

/* AUTOSAVETIME FRAME */
   autosavetimeFrame = XtVaCreateWidget("autosavetime", xmFrameWidgetClass, configFO,
			XmNrightAttachment, XmATTACH_FORM,
			XmNtopAttachment,   XmATTACH_WIDGET,
			XmNtopWidget,       controlFrame,
			XmNtopOffset,       5,
			XmNleftAttachment,  XmATTACH_FORM,
			NULL);

/* PRINTER FRAME */
   printerFrame = XtVaCreateWidget("printer", xmFrameWidgetClass, configFO,
			XmNtopWidget,       autosavetimeFrame,
			XmNtopAttachment,   XmATTACH_WIDGET,
			XmNleftAttachment,  XmATTACH_FORM,
			XmNrightAttachment, XmATTACH_FORM,
			NULL);

/********************************************************************************/
/*      OBJECT DEFINITIONS                                                      */
/********************************************************************************/

/* PRINTER LABEL WIDGET */
   XtVaCreateManagedWidget("printer", xmLabelGadgetClass, printerFrame,
			XmNchildType, XmFRAME_TITLE_CHILD,
			NULL);

/* PRINTER TEXT */
   printerText = XtVaCreateManagedWidget("printerText", xmTextFieldWidgetClass, printerFrame,
			XmNvalue,     DefaultPrinter,
			XmNmaxLength, 20,
			XmNcolumns,   15,
			NULL);
   XtAddCallback(printerText, XmNlosingFocusCallback, gui_printerTextCB, NULL);

   XtManageChild(printerFrame);

/*****************************************************************************/

/* AUTOSAVETIME ROW COLUMN */
   autosavetimeFO = XtVaCreateWidget("Time", xmRowColumnWidgetClass, autosavetimeFrame,
			XmNnumColumns,  3,
			XmNorientation, XmHORIZONTAL,
			XmNspacing,     3,
			NULL);

/* AUTOSAVETIME LABEL WIDGET */
   XtVaCreateManagedWidget("asave_time1", xmLabelGadgetClass, autosavetimeFO, NULL);

/* AUTOSAVETIME TEXT */
   sprintf(txt, "%d", AutoSaveTime);
   autosavetimeText = XtVaCreateManagedWidget("autosavetimeText", xmTextFieldWidgetClass, autosavetimeFO,
			XmNvalue,     txt,
			XmNmaxLength, 20,
			XmNcolumns,   5,
			NULL);
   XtAddCallback(autosavetimeText, XmNlosingFocusCallback, gui_autosavetimeTextCB, NULL);

/* AUTOSAVETIME LABEL 2 WIDGET */
   XtVaCreateManagedWidget("asave_time2", xmLabelGadgetClass, autosavetimeFO, NULL);

   XtManageChild(autosavetimeFO);
   XtManageChild(autosavetimeFrame);

/**************************************************************************************/

/* LABEL FOR CHECK-BOX */
   CheckLabel = XtVaCreateManagedWidget("CheckLabel", xmLabelGadgetClass, onOffFrame,
			XmNchildType, XmFRAME_TITLE_CHILD,
			NULL);

/* CREATE CHECK-BOX */
   n = 0;
   XtSetArg(args[n], XmNnumColumns, 1); n++;
   XtSetArg(args[n], XmNspacing,    3); n++;
   optionCheck = XmCreateSimpleCheckBox(onOffFrame, "optionCheck", args, n);

/* ON OFF BUTTONS */
   autosaveTB = XtVaCreateManagedWidget("asave", xmToggleButtonWidgetClass, optionCheck, NULL);
   XtAddCallback(autosaveTB, XmNvalueChangedCallback, gui_toggleItemCB, (XtPointer)&AutoSave);

   backupTB = XtVaCreateManagedWidget("backup", xmToggleButtonWidgetClass, optionCheck, NULL);
   XtAddCallback(backupTB, XmNvalueChangedCallback, gui_toggleItemCB, (XtPointer)&AutoBackup);

   autoindTB = XtVaCreateManagedWidget("autoind", xmToggleButtonWidgetClass, optionCheck, NULL);
   XtAddCallback(autoindTB, XmNvalueChangedCallback, gui_toggleItemCB, (XtPointer)&AutoIndent);

   wordwrapTB = XtVaCreateManagedWidget("wordwrap", xmToggleButtonWidgetClass, optionCheck, NULL);
   XtAddCallback(wordwrapTB, XmNvalueChangedCallback, gui_toggleItemCB, (XtPointer)&WordWrap);

   fileboxTB = XtVaCreateManagedWidget("filebox", xmToggleButtonWidgetClass, optionCheck, NULL);
   XtAddCallback(fileboxTB, XmNvalueChangedCallback, gui_toggleItemCB, (XtPointer)&FileBox);

   XtManageChild(optionCheck);
   XtManageChild(onOffFrame);


/* MAIN CONTROL BUTTONS */
   controlFO = XtVaCreateWidget("controlbox", xmRowColumnWidgetClass, controlFrame,
			XmNnumColumns,     1,
			XmNpacking,        XmPACK_COLUMN,
			XmNentryAlignment, XmALIGNMENT_CENTER,
			XmNisAligned,      True,
			NULL);

/* OK BUTTON WIDGET */
   button = XtVaCreateManagedWidget("configOk", xmPushButtonGadgetClass, controlFO, NULL);
   XtAddCallback(button, XmNdisarmCallback, gui_okConfigCB, NULL);

/* SAVE CONFIG BUTTON WIDGET */
   button = XtVaCreateManagedWidget("configSave", xmPushButtonGadgetClass, controlFO, NULL);
   XtAddCallback(button, XmNdisarmCallback, gui_saveConfigCB, NULL);

/* CANCEL BUTTON WIDGET */
   button = XtVaCreateManagedWidget("configCancel", xmPushButtonGadgetClass, controlFO, NULL);
   XtAddCallback(button, XmNdisarmCallback, gui_cancelConfigCB, NULL);

/* RESTORE DEFAULTS BUTTON WIDGET */
   button = XtVaCreateManagedWidget("configRestore", xmPushButtonGadgetClass, controlFO, NULL);
   XtAddCallback(button, XmNdisarmCallback, gui_restoreConfigCB, NULL);

/* HELP BUTTON WIDGET */
   button = XtVaCreateManagedWidget("configHelp", xmPushButtonGadgetClass, controlFO, NULL);
   XtAddCallback(button, XmNdisarmCallback, gui_helpCB, (XtPointer)HELPSETUP);

   XtManageChild(controlFO);

   XtManageChild(controlFrame);

   XtManageChild(configFO);
}
