/* Copyright (C) 1994 Queen Mary and Westfield College, University of London

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   A copy of the GNU General Public License may be found in
   doc/COPYING in the source tree of this programm.

   This file was written by: Franz Gerschwiler.(goblin@dcs.qmw.ac.uk)

 ****

   modified by: p.hlstede	(peter@calvados.chemie.uni-bremen.de)

   - the code was reorganized and ansified
   - Motif 1.2 functions can be used for searching and replacing instead
     of doing all by hand

							06.09.1997
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <Xm/Xm.h>
#include <Xm/Text.h>

#include "sys_consts.h"
#include "globals.h"
#include "utils.h"
#include "callbacks.h"

#include "dstring.h"


/***  GLOBAL VARIABLES  ***/
Boolean  SelectedOnly;

/***  Local prototypes  ***/
static int  gui_search(void);
static int  calculateRowHeight(void);
static void replaceSelection(char *word, char *replacement);
static void replaceText(char *word, char *replacement);


/* Try to calculate the height of the text rows in the editor. */
/* The original version of gloe used the fixed value 15 for this. */
static int  calculateRowHeight(void)
{
   int  base = XmTextGetBaseline (text_canvas);
   Dimension s, m, h;
   XtVaGetValues(text_canvas,
		XmNmarginHeight,      &m,
		XmNshadowThickness,   &s,
		XmNhighlightThickness, &h,
		NULL);
   return(base - s - h - m + 1);
}

/* Jump to a line selected by the user */
void  gui_jumpCB(Widget w, XtPointer client_data, XtPointer call_data)
{
   char      *text;
   char      *temp;
   int        linenum;
   int        linecount;
   int        charcount, charcount2;
   int        textlength;
   Dimension  height;
   int        rows;

   /* Initialize Variables */
   linecount = 1;
   charcount = charcount2 = -1;

   /* Get data from jump_to text */
   temp = XmTextGetString(jumpMenuText);
   linenum = -1;
   if (temp)
   {
      if (*temp)
         linenum = atoi(temp);
      XtFree(temp);
   }
   if (linenum < 0)
      return;

   /* Get data from text canvas */
   text = XmTextGetString(text_canvas);
   textlength = strlen(text);
   XtVaGetValues(text_canvas, XmNheight, &height, NULL);
   rows = (int)height / calculateRowHeight();

   /* Nothing sophisticated, each '\n' represents a new line */
   while ((charcount < textlength) && (linecount < linenum))
   {
      charcount++;
      if (text[charcount] == '\n')
	 linecount++;
      if (linecount == linenum - (int)rows / 2)
	 charcount2 = charcount + 1;
   };
   charcount++;

   /* Check if line was found */
   if (linecount != linenum)
   {
      /* Tell user number invalid */ ;
      set_status_message("sorry line number invalid");
   }
   else
   {
      /* Set cursor to beginning of line  */
      XmTextSetTopCharacter(text_canvas, (XmTextPosition)charcount2);
      XmTextSetInsertionPosition(text_canvas, (XmTextPosition)charcount);
      set_status_message("Ok jumping to line");
   }

   XtFree(text);
}



void  gui_searchReplaceCB(Widget w, XtPointer client_data, XtPointer call_data)
{
   /* Decide what sort of search request occured */
   switch ((int)client_data)
   {
      case SEARCH:				/* Request to search for a word */
	 set_cursor(toplevel, TimeCursor);
	 set_cursor(text_canvas, TimeCursor);
	 if (gui_search() > 0)
	 {
	    set_status_message("Found an instance");

	    /* Now allow Replace to occur */
	    XtSetSensitive(replaceMenuPB, True);
	 }
	 else
	    set_status_message("Sorry couldn't find the word");

	 set_cursor(toplevel, MainCursor);
	 set_cursor(text_canvas, TextCursor);
	 break;

      case REPLACE:		/* Request to replace a word */
	 /* FIRST TAKE A SNAPSHOT TO ALLOW FOR UNDO */
	 gui_replaceCB((Widget) 0, NULL, NULL);
	 XtSetSensitive(replaceMenuPB, False);
	 break;

      case REPLACEALL:		/* Request for replace all action */
	 set_cursor(toplevel, TimeCursor);
	 set_cursor(text_canvas, TimeCursor);
	 gui_replaceAllCB((Widget) 0, NULL, NULL);
	 XtSetSensitive(replaceMenuPB, False);
	 set_cursor(toplevel, MainCursor);
	 set_cursor(text_canvas, TextCursor);
	 break;
   }
}

/* Turn search within selction only either on or off */
void  searchSelectionOnlyToggleCB(Widget w, XtPointer client_data, XtPointer call_data)
{
   XtVaGetValues(w, XmNset, &SelectedOnly, NULL);
}

#ifndef LET_MOTIF_SEARCH
static int  gui_search(void)
{
   char     *text = NULL;
   char     *base = NULL;
   char     *word = NULL;
   char     *instanceofwordptr = NULL;
   int       pos, wordpos, rows, linecount, charcount;
   Dimension height;

   /* Get word to be searched for */
   word = XmTextGetString(searchMenuText);
   if (!word || !*word)
   {
      XtFree(word);
      return(0);
   }

   pos = XmTextGetInsertionPosition(text_canvas);
   base = XmTextGetString(text_canvas);
   text = base + pos;

   /* Search from cursor position down */
   instanceofwordptr = strstr(text, word);
   if (instanceofwordptr == NULL)
   {
      /* No luck, ok search from beginning */
      instanceofwordptr = strstr(base, word);
      if (instanceofwordptr == NULL)
      {
	 XtFree(base);
	 XtFree(word);
	 return(0);
      }
   }

   /* Ok there is an instance so now locate it in the canvas */
   wordpos = (instanceofwordptr - base);

   /* Now motif doesn't offer any thing to return the height */
   /* in columns of the text canvas so I'm going to have to guestimate */
   XtVaGetValues(text_canvas, XmNheight, &height, NULL);
   rows = (int)height / calculateRowHeight();

   /* Now search backwards rows/2 lines */
   charcount = wordpos;
   linecount = (int)rows / 2;
   while ((linecount > 0) && (charcount > 1))
   {
      charcount--;
      if (base[charcount] == '\n')
         linecount--;
   }
   XmTextSetTopCharacter(text_canvas, charcount);

   /* Now set insertion position to beggining of word */
   XmTextSetInsertionPosition(text_canvas, (XmTextPosition)(wordpos + strlen(word)));

   /* Highlight the word */
   XmTextSetSelection(text_canvas, (XmTextPosition)wordpos, (XmTextPosition)(wordpos + strlen(word)), CurrentTime);

   XtFree(word);
   XtFree(base);

   return(1);
}
#else
static int  gui_search(void)
{
   char            *word = NULL;
   XmTextPosition   pos, wordpos;

   /* Get word to be searched for */
   word = XmTextGetString(searchMenuText);
   if (!word || !*word)
   {
      XtFree(word);
      return(0);
   }

   /* first search from the actual position and if the string wasn't found, */
   /* try again from the start */
   pos = XmTextGetInsertionPosition(text_canvas);
   if (!XmTextFindString(text_canvas, pos, word, XmTEXT_FORWARD, &wordpos) &&
       !XmTextFindString(text_canvas, 0, word, XmTEXT_FORWARD, &wordpos))
   {
      XtFree(word);
      return(0);
   }

/*   XmTextShowPosition(text_canvas, wordpos);*/

   /* Now set insertion position to beginning of word */
   XmTextSetInsertionPosition(text_canvas, (XmTextPosition)(wordpos + strlen(word)));

   /* Highlight the word */
   XmTextSetSelection(text_canvas, wordpos, (XmTextPosition)(wordpos + strlen(word)), CurrentTime);

   XtFree(word);

   return(1);
}
#endif

void  gui_replaceCB(Widget w, XtPointer client_data, XtPointer call_data)
{
   char    *word;
   char    *replacement;
   char    *temp;
   XmTextPosition pos, firstpos;

   /* Get data from widgets */
   word = XmTextGetString(searchMenuText);
   replacement = XmTextGetString(replaceMenuText);
   temp = XmTextGetSelection(text_canvas);
   if (strcmp(temp, word) == 0)
   {
      XmTextGetSelectionPosition(text_canvas, &firstpos, &pos);

      /* Now perform replacement */
      XmTextReplace(text_canvas, firstpos, pos, replacement);

      /* Now place the cursor at the end of the word */
      XmTextSetInsertionPosition(text_canvas, (XmTextPosition) (firstpos + strlen(replacement)));

      /* Now turn Replace button off again */
      XtSetSensitive(replaceMenuPB, False);
   }
   else
      set_status_message("Sorry can't replace, ...you lost the selection");

   XtFree(word);
   XtFree(replacement);
   XtFree(temp);
}


void  gui_replaceAllCB(Widget w, XtPointer client_data, XtPointer call_data)
{
   char    *word;
   char    *replacement;

   /* Get data from widgets */
   word = XmTextGetString(searchMenuText);
   replacement = XmTextGetString(replaceMenuText);

   if (!*word)
      set_status_message("Sorry No search text specified");
   else
   {
      if (strcmp(word, replacement) != 0)
      {
         if (SelectedOnly)
	    replaceSelection(word, replacement);
	 else
	    replaceText(word, replacement);
      }
      else
          set_status_message("They're the same why bother");
   }

   XtFree(word);
   XtFree(replacement);
}

static void replaceSelection(char *word, char *replacement)
{
   char    *temp;
   char    *temp1;
   DString  final;
   char    *instanceptr;
   XmTextPosition left, right;

   DStringInit(&final);
   temp = XmTextGetSelection(text_canvas);
   if (temp && *temp)
   {
      set_status_message("Ok replacing instances within selection");
      XmTextGetSelectionPosition(text_canvas, &left, &right);
      temp1 = temp;

      /* Now do search and replace on string */
      instanceptr = strstr(temp1, word);
      if (instanceptr != NULL)
      {
         while (instanceptr != NULL)
	 {
	    DStringNCat(&final, temp1, instanceptr - temp1);
	    DStringCat(&final, replacement);
	    temp1 = instanceptr + strlen(word);
	    instanceptr = strstr(temp1, word);
	 }
         DStringCat(&final, temp1);
         XmTextReplace(text_canvas, left, right, final.str);
         XmTextSetSelection(text_canvas, left, left + strlen(final.str), CurrentTime);
         set_status_message("Done replacing instances within selection");
      }
      else
      {
         set_status_message("Sorry no instances found");
      }
   }
   else
   {
      set_status_message("Sorry no selection to edit");
   }
   XtFree(temp);
   DStringFree(&final);
}

#ifndef LET_MOTIF_SEARCH
static void replaceText(char *word, char *replacement)
{
   char    *temp;
   char    *temp1;
   DString  final;
   char    *instanceptr;

   DStringInit(&final);
   set_status_message("Ok replacing instances");
   temp = XmTextGetString(text_canvas);
   temp1 = temp;
   instanceptr = strstr(temp1, word);
   if (instanceptr != NULL)
   {
      while (instanceptr != NULL)
      {
	 DStringNCat(&final, temp1, instanceptr - temp1);
	 DStringCat(&final, replacement);
	 temp1 = instanceptr + strlen(word);
	 instanceptr = strstr(temp1, word);
      }
      DStringCat(&final, temp1);
      XmTextSetString(text_canvas, final.str);
      set_status_message("Done replacing instances");
   }
   else
   {
      set_status_message("Sorry no instances found");
   }
   XtFree(temp);
   DStringFree(&final);
}
#else
static void replaceText(char *word, char *replacement)
{
   XmTextPosition  start, found;
   int             wordlen = strlen(word);
   int             replen  = strlen(replacement);

   start = 0;
   while (XmTextFindString(text_canvas, start, word, XmTEXT_FORWARD, &found))
   {
      XmTextReplace(text_canvas, found, found + wordlen, replacement);
      start = found + replen;
   }
}
#endif

void  gui_matchCB(Widget w, XtPointer client_data, XtPointer call_data)
{
   char    *parentheses;
   char    *temp;
   int      matchChar;
   int      up, down, index;
   XmTextPosition left, right;

   parentheses = XmTextGetSelection(text_canvas);
   temp = XmTextGetString(text_canvas);
   XmTextGetSelectionPosition(text_canvas, &left, &right);

   if (parentheses != NULL)
   {
      if (strlen(parentheses) == 1)
      {
	 matchChar = '\0';
	 set_status_message("Ok matching");
	 if (parentheses[0] == '(')
	    matchChar = ')';
	 if (parentheses[0] == '{')
	    matchChar = '}';
	 if (parentheses[0] == '<')
	    matchChar = '>';
	 if (parentheses[0] == ')')
	    matchChar = '(';
	 if (parentheses[0] == '}')
	    matchChar = '{';
	 if (parentheses[0] == '>')
	    matchChar = '<';

	 if ((matchChar == ')') || (matchChar == '}') || (matchChar == '>'))
	 {
	    /* Search forwards */
	    index = (int)left + 1;
	    up = 1;
	    down = 0;
	    while (up != down && (index <= strlen(temp) - 1))
	    {
	       if (temp[index] == parentheses[0])
		  up++;
	       if (temp[index] == matchChar)
		  down++;
	       index++;
	    }
	    if (up == down)
	       XmTextSetSelection(text_canvas, (XmTextPosition) left, (XmTextPosition) index, CurrentTime);
	    else
	       set_status_message("Sorry! No match found");
	 }
	 else
	 {
	    if ((matchChar == '(') || (matchChar == '{') || (matchChar == '<'))
	    {
	       index = (int)left - 1;
	       up = 1;
	       down = 0;
	       while (up != down && (index >= 0))
	       {
		  if (temp[index] == parentheses[0])
		     up++;
		  if (temp[index] == matchChar)
		     down++;
		  index--;
	       }
	       if (up == down)
		  XmTextSetSelection(text_canvas, (XmTextPosition) index + 1, (XmTextPosition) left + 1, CurrentTime);
	       else
		  set_status_message("Sorry! No match found");
	    }
	    else
	       set_status_message("Sorry! No match found");
	 }
      }
      else
	 set_status_message("Sorry! The selection is ambigous");
   }
   XtFree(temp);
   XtFree(parentheses);
}
